﻿
using System;
using System.Threading;
using System.Diagnostics;
using System.Windows.Media;
using System.Linq;

using Styx.Common;
using Styx.Common.Helpers;
using Styx.Plugins;
using Styx.WoWInternals;




namespace Roller
{
    public partial class Roller : HBPlugin
    {
        private static RollerGUI GUI = new RollerGUI();
        LuaEventArgs oldArgs = null;
        public const int Need = 1;
        public const int Greed = 2;
        public const int Dez = 3;
        public const int Pass = 0;

    #region Overrides of HBPlugin 
        public override void Initialize()
        {
            Slog("Version " + Version +  " loaded");
            Lua.Events.AttachEvent("START_LOOT_ROLL", CheckQuality);
            Lua.Events.AttachEvent("CONFIRM_LOOT_ROLL", ConfirmRoll);
            Lua.Events.AttachEvent("CONFIRM_DISENCHANT_ROLL", ConfirmRoll);

        }

        public override void Dispose()
        {
        }

        public override string  ButtonText { get { return "Set"; } }
        public override bool    WantButton { get { return true; } }
        public override void OnButtonPress() {GUI.ShowDialog(); }
        private readonly WaitTimer timer0 = WaitTimer.TenSeconds;
        public override string  Name { get { return "Roller"; } }
        public override string  Author { get { return "Powa"; } }
        public override Version Version { get { return new Version(0, 28); } }

        public override void Pulse() {}

    #endregion

        /*  ItemQuality
         *  0 - Grey
            1 - White
            2 - Green
            3 - Rare
            4 - Epic
            5 - Legendary

         * 
            ItemInfo
         *  0 - texture (string)
            1 - name (string)
            2 - count (integer)
            3 - quality (integer)
            4 - bindOnPickUp (boolean)
            5 - canNeed (boolean)
            6 - canGreed (boolean)
            7 - canDisenchant (boolean)
         * 
            RollType
            0 - Pass
            1 - Need 
            2 - Greed
            3 - Disenchant
        */



        private void CheckQuality(object sender, LuaEventArgs e)
        {
   
            //Events seems to fire twice, temporary fix
            if (e == oldArgs)
                return;
            else
                oldArgs = e;	
            
            // Don't run anything if the plugin is not enabled
            var plugin = PluginManager.Plugins.FirstOrDefault(p => p.Name == Name);
            if ((plugin != null && !plugin.Enabled)|| e.Args == null)
				return;

            //getting item info
            double rollId = (double)e.Args[0];   
            bool canNeed = Lua.GetReturnVal<bool>("return GetLootRollItemInfo(" + rollId + ")", 5);
            bool canGreed = Lua.GetReturnVal<bool>("return GetLootRollItemInfo(" + rollId + ")", 6);
            bool canDez = Lua.GetReturnVal<bool>("return GetLootRollItemInfo(" + rollId + ")", 7);
            int itemQuality = Lua.GetReturnVal<int>("return GetLootRollItemInfo(" + rollId + ")", 3);
            string itemName = Lua.GetReturnVal<string>("return GetLootRollItemInfo(" + rollId + ")", 1);
           

            switch(itemQuality)
            {
                case 2 :    //Green
                    if (canDez && GUI.greenDez.Checked)
                        Roll(Dez, rollId, itemName);
                    else if (canGreed && GUI.greenGreed.Checked)
                        Roll(Greed, rollId, itemName);
                    else if (canNeed && GUI.greenNeed.Checked)
                        Roll(Need, rollId, itemName);
                    else if (GUI.greenPass.Checked)
                        Roll(Pass, rollId, itemName);
                    else
                    {
                        cantRoll(itemQuality, itemName, rollId);
                        Roll(Greed, rollId, itemName);
                    }
                    break;
               
                case 3:    //Blue
                    if (canDez && GUI.blueDez.Checked)
                        Roll(Dez, rollId, itemName);
                    else if (canGreed && GUI.blueGreed.Checked)
                        Roll(Greed, rollId, itemName);
                    else if (canNeed && GUI.blueNeed.Checked)
                        Roll(Need, rollId, itemName);
                    else if (GUI.bluePass.Checked)
                        Roll(Pass, rollId, itemName);
                    else
                    {
                        cantRoll(itemQuality, itemName, rollId);
                        Roll(Greed, rollId, itemName);
                    }
                    break;

                case 4:    //Epic
                    if (canDez && GUI.epicDez.Checked)
                        Roll(Dez, rollId, itemName);
                    else if (canGreed && GUI.epicGreed.Checked)
                        Roll(Greed, rollId, itemName);
                    else if (canNeed && GUI.epicNeed.Checked)
                        Roll(Need, rollId, itemName);
                    else if (GUI.epicPass.Checked)
                        Roll(Pass, rollId, itemName);
                    else
                    {
                        cantRoll(itemQuality, itemName, rollId);
                        Roll(Greed, rollId, itemName);
                    }
                    break;

                case 5:    //Legendary
                    if (canDez && GUI.legendaryDez.Checked)
                        Roll(Dez, rollId, itemName);
                    else if (canGreed && GUI.legendaryGreed.Checked)
                        Roll(Greed, rollId, itemName);
                    else if (canNeed && GUI.legendaryNeed.Checked)
                        Roll(Need, rollId, itemName);
                    else if (GUI.legendaryPass.Checked)
                        Roll(Pass, rollId, itemName);
                    else
                    {
                        cantRoll(itemQuality, itemName, rollId);
                        Roll(Greed, rollId, itemName);
                    }
                    break;

                default:
                    //Slog("Problem with Item Quality");
                    Roll(Greed, rollId, itemName);
                    break;
            }

        }


        private void cantRoll(int itemQuality, string itemName, double rollId)
        {

            string msg = "Can't roll ";
            switch (itemQuality)
            {
                case 2:    //Green
                    if (GUI.greenDez.Checked)
                        msg+="Disenchant ";
                    else if (GUI.greenGreed.Checked)
                        msg+="Greed ";
                    else if (GUI.greenNeed.Checked)
                        msg+="Need ";
                    else if (GUI.greenPass.Checked)
                        msg+="Pass ";
                    break;

                case 3:    //Blue
                    if (GUI.blueDez.Checked)
                        msg+="Disenchant ";
                    else if (GUI.blueGreed.Checked)
                        msg+="Greed ";
                    else if (GUI.blueNeed.Checked)
                        msg+="Need ";
                    else if (GUI.bluePass.Checked)
                        msg+="Pass ";
                    break;

                case 4:    //Epic
                    if (GUI.epicDez.Checked)
                        msg+="Disenchant ";
                    else if (GUI.epicGreed.Checked)
                        msg+="Greed ";
                    else if (GUI.epicNeed.Checked)
                        msg+="Need ";
                    else if (GUI.epicPass.Checked)
                        msg+="Pass ";
                    break;

                case 5:    //Legendary
                    if (GUI.legendaryDez.Checked)
                        msg+="Disenchant ";
                    else if (GUI.legendaryGreed.Checked)
                        msg+="Greed ";
                    else if (GUI.legendaryNeed.Checked)
                        msg+="Need ";
                    else if (GUI.legendaryPass.Checked)
                        msg+="Pass ";
                    break;

            }
            msg+=("on "+itemName);

            Slog(msg);
        
        
        
        }

        private void Roll(int rollType, double rollId, string itemName)
        {
            PauseMilli(2000);
            if (itemName.Equals(""))
                return;
                
            switch (rollType)
            {
                case 0:    //Pass
                    Slog("Rolling Pass on: " + itemName);
                    break;

                case 1:    //Need
                    Slog("Rolling Need on: " + itemName);
                    break;        

                case 2:    //Greed
                    Slog("Rolling Greed on: " + itemName);
                    break;

                case 3:    //Disenchant
                    Slog("Rolling Disenchant on: " + itemName);
                    break;

                default:
                    Slog("Problem while rolling - Rolling Greed on: " + itemName);
                    rollType = 2;   //Rolling Greed
                    break;
            }

            Lua.DoString("RollOnLoot(" + rollId + ", "+ rollType + ")");
        }

        private void ConfirmRoll(object sender, LuaEventArgs e)
        {
            Slog("Confirming Loot Roll");
            PauseMilli(1000);

            double rollId = (double)e.Args[0];
            double rollType = (double)e.Args[1];

            Lua.DoString("ConfirmLootRoll({0},{1})", rollId, rollType);
        }

        public static void PauseMilli(int howLong)
        {
            Stopwatch myTimer = new Stopwatch();
            Random rnd = new Random();
			int length = rnd.Next(howLong, howLong + 1500);
			
            myTimer.Restart();    

            while (myTimer.ElapsedMilliseconds <= length);     //do nothing
            
            myTimer.Stop();
             
            
        }
        
        public static void Slog(string msg)
        {
           Logging.Write(Colors.Yellow, "[Roller] "+ msg);   
        }
    }
}